svn commit: r314199 - stable/10/libexec/rtld-elf

Konstantin Belousov kib at FreeBSD.org
Fri Feb 24 11:07:50 UTC 2017


Author: kib
Date: Fri Feb 24 11:07:49 2017
New Revision: 314199
URL: https://svnweb.freebsd.org/changeset/base/314199

Log:
  MFC r313494:
  Handle protected symbols in rtld.

Modified:
  stable/10/libexec/rtld-elf/rtld.c
Directory Properties:
  stable/10/   (props changed)

Modified: stable/10/libexec/rtld-elf/rtld.c
==============================================================================
--- stable/10/libexec/rtld-elf/rtld.c	Fri Feb 24 10:20:13 2017	(r314198)
+++ stable/10/libexec/rtld-elf/rtld.c	Fri Feb 24 11:07:49 2017	(r314199)
@@ -3833,15 +3833,19 @@ symlook_default(SymLook *req, const Obj_
     donelist_init(&donelist);
     symlook_init_from_req(&req1, req);
 
-    /* Look first in the referencing object if linked symbolically. */
-    if (refobj->symbolic && !donelist_check(&donelist, refobj)) {
-	res = symlook_obj(&req1, refobj);
-	if (res == 0) {
-	    req->sym_out = req1.sym_out;
-	    req->defobj_out = req1.defobj_out;
-	    assert(req->defobj_out != NULL);
-	}
+    /*
+     * Look first in the referencing object if linked symbolically,
+     * and similarly handle protected symbols.
+     */
+    res = symlook_obj(&req1, refobj);
+    if (res == 0 && (refobj->symbolic ||
+      ELF_ST_VISIBILITY(req1.sym_out->st_other) == STV_PROTECTED)) {
+	req->sym_out = req1.sym_out;
+	req->defobj_out = req1.defobj_out;
+	assert(req->defobj_out != NULL);
     }
+    if (refobj->symbolic || req->defobj_out != NULL)
+	donelist_check(&donelist, refobj);
 
     symlook_global(req, &donelist);
 


More information about the svn-src-stable mailing list